@@ -101,6 +101,20 @@ void Pipe::rotate(bool clockwise)
     }
 }
 
+std::vector<Side> Pipe::connections() const
+{
+    std::vector<Side> sides;
+    sides.reserve(Side::max);
+
+    for (const Side s : { Side::Top, Side::Right, Side::Bottom, Side::Left }) {
+        if (get(s)) {
+            sides.push_back(s);
+        }
+    }
+
+    return sides;
+}
+
 bool Pipe::get(const Side& side) const
 {
     return sides[side];
@@ -135,16 +149,16 @@ Pipe::operator Type() const
 
 Cell::Status Cell::update()
 {
-    if (rotate_start) {
+    if (rotation()) {
         const Uint64 diff = SDL_GetTicks64() - rotate_start;
-        if (diff > rotation_time) {
+        if (diff >= rotation_time) {
             // rotation completed
             rotate_start = 0;
             if (rotate_twice) {
-                rotate_twice = false;
                 rotate(rotate_clockwise);
+            } else {
+                return Cell::RotationComplete;
             }
-            return Cell::RotationComplete;
         }
         return Cell::RotationInProgress;
     }
@@ -152,30 +166,41 @@ Cell::Status Cell::update()
     return Cell::Unchanged;
 }
 
-double Cell::angle() const
+double Cell::phase() const
 {
-    double angle = pipe.angle();
+    double phase = 1.0;
 
-    if (rotate_start) {
+    if (rotation()) {
         const Uint64 diff = SDL_GetTicks64() - rotate_start;
         if (diff < rotation_time) {
-            const float diff_angle = static_cast<float>(diff) /
-                static_cast<float>(rotation_time) * 90.0f;
-            angle = rotate_pipe.angle();
-            if (rotate_clockwise) {
-                angle += diff_angle;
-            } else {
-                angle -= diff_angle;
-            }
+            phase =
+                static_cast<double>(diff) / static_cast<double>(rotation_time);
         }
     }
 
+    return phase;
+}
+
+double Cell::angle() const
+{
+    double angle;
+
+    if (!rotation()) {
+        angle = pipe.angle();
+    } else {
+        angle = phase() * 90.0;
+        if (!rotate_clockwise) {
+            angle = -angle;
+        }
+        angle += rotate_pipe.angle();
+    }
+
     return angle;
 }
 
 void Cell::rotate(bool clockwise)
 {
-    if (rotate_start) {
+    if (rotation()) {
         // rotation in progress
         if (rotate_clockwise == clockwise) {
             rotate_twice = true;
@@ -192,6 +217,7 @@ void Cell::rotate(bool clockwise)
             rotate_clockwise = clockwise;
         }
     } else {
+        rotate_twice = false;
         rotate_pipe = pipe;
         rotate_clockwise = clockwise;
         rotate_start = SDL_GetTicks64();
