@@ -6,6 +6,8 @@
 
 #include "buildcfg.h"
 
+#include <cmath>
+
 struct LevelSize {
     size_t size;
     const char* name;
@@ -19,6 +21,7 @@ static const LevelSize level_sizes[] = {
 
 Game::Game(SDL_Window* wnd, SDL_Renderer* renderer)
     : window(wnd)
+    , layout()
     , render(renderer)
     , puzzle_mode(true)
 {
@@ -110,7 +113,7 @@ void Game::handle_event(const SDL_Event& event)
     }
 }
 
-void Game::update()
+bool Game::update()
 {
     level.update();
 
@@ -128,6 +131,8 @@ void Game::update()
     if (!level.state.level_complete && level.state.rotation_complete) {
         sound.play(Sound::Clatz);
     }
+
+    return level.state.rotation_active || level.state.level_complete;
 }
 
 void Game::draw()
@@ -172,6 +177,7 @@ void Game::draw_puzzle()
 
     // pipes shadow
     const int shadow_shift = layout.cell_size / 20;
+    const int lift_shift = layout.cell_size / 16;
     for (size_t y = 0; y < level.height; ++y) {
         for (size_t x = 0; x < level.width; ++x) {
             const Cell& cell = level.get_cell({ x, y });
@@ -194,6 +200,11 @@ void Game::draw_puzzle()
                 default:
                     continue;
             }
+            if (cell.rotation()) {
+                const int shift = lift_shift * sin(M_PI * cell.phase());
+                dst.x += shift;
+                dst.y += shift;
+            }
             render.draw(tid, dst, cell.angle(), 0.3);
         }
     }
@@ -224,6 +235,11 @@ void Game::draw_puzzle()
                 default:
                     continue;
             }
+            if (cell.rotation()) {
+                const int shift = lift_shift * sin(M_PI * cell.phase());
+                dst.x -= shift;
+                dst.y -= shift;
+            }
             render.draw(tid, dst, cell.angle());
         }
     }
@@ -438,12 +454,7 @@ void Game::reset_level(bool regen)
         layout.update(level.width, level.height);
     }
 
-#ifdef DEBUG
-    level.get_cell(level.sender).rotate(false);
-#else
     level.reset();
-#endif // DEBUG
-
     level.update();
 }
 
