@@ -15,8 +15,8 @@ void Level::generate()
     std::fill(cells.begin(), cells.end(), Cell {});
 
     // install sender (server)
-    sender.x = mtrand::get(1ul, width - 1);
-    sender.y = mtrand::get(1ul, height - 1);
+    sender.x = mtrand::get(static_cast<size_t>(1), width - 1);
+    sender.y = mtrand::get(static_cast<size_t>(1), height - 1);
     get_cell(sender).object = Cell::Sender;
 
     const size_t max_recievers = cells.size() / 5;
@@ -63,6 +63,7 @@ void Level::update()
 {
     // update cells state
     state.rotation_complete = false;
+    state.rotation_active = false;
     Position pos;
     for (pos.y = 0; pos.y < height; ++pos.y) {
         for (pos.x = 0; pos.x < width; ++pos.x) {
@@ -73,14 +74,16 @@ void Level::update()
                     state.rotation_complete = true;
                     break;
                 case Cell::Unchanged:
+                    break;
                 case Cell::RotationInProgress:
+                    state.rotation_active = true;
                     break;
             }
         }
     }
 
     // trace from sender
-    if (get_cell(sender).rotate_start == 0) {
+    if (!get_cell(sender).rotation()) {
         trace_state(sender);
     }
 
@@ -146,7 +149,8 @@ void Level::add_reciever()
     }
 
     // get random position
-    const size_t free_index = mtrand::get(0ul, free_cells.size());
+    const size_t free_index =
+        mtrand::get(static_cast<size_t>(0), free_cells.size());
     const Position& reciever = free_cells[free_index];
 
     // find path from receiver to sender
@@ -176,9 +180,9 @@ bool Level::find_path(const Position& from, std::set<Position>& vizited,
         sides.push_back(Side::Top);
         sides.push_back(Side::Bottom);
         for (size_t i = 0; i < 4; ++i) {
-            const size_t index0 = mtrand::get(0ul, sides.size());
-            const size_t index1 = mtrand::get(0ul, sides.size());
-            std::iter_swap(sides.begin() + index0, sides.begin() + index1);
+            const size_t i0 = mtrand::get(static_cast<size_t>(0), sides.size());
+            const size_t i1 = mtrand::get(static_cast<size_t>(0), sides.size());
+            std::iter_swap(sides.begin() + i0, sides.begin() + i1);
         }
     } else {
         // shortest path
@@ -251,21 +255,33 @@ void Level::apply_path(const Position& start, const Level::Path& path)
     }
 }
 
-void Level::trace_state(const Position& pos)
+void Level::trace_state(const Position& start)
 {
-    Cell& cell = get_cell(pos);
-    const Side sides[] = { Side::Top, Side::Right, Side::Bottom, Side::Left };
+    Position curr_pos = start;
 
-    for (const Side side : sides) {
-        if (cell.pipe.get(side)) {
-            const Position next_pos = neighbor(pos, side);
-            Cell& next_cell = get_cell(next_pos);
-            if (next_cell.rotate_start == 0 && !next_cell.active &&
-                next_cell.pipe.get(side.opposite())) {
-                next_cell.active = true;
-                trace_state(next_pos);
+    while (true) {
+        Cell& curr_cell = get_cell(curr_pos);
+        curr_cell.active = true;
+
+        // get possible ways
+        std::vector<Position> connected;
+        for (const Side side : curr_cell.pipe.connections()) {
+            const Position next_pos = neighbor(curr_pos, side);
+            const Cell& next_cell = get_cell(next_pos);
+            if (next_pos != curr_pos && !next_cell.rotation() &&
+                !next_cell.active && next_cell.pipe.get(side.opposite())) {
+                connected.push_back(next_pos);
             }
         }
+        if (connected.empty()) {
+            break;
+        }
+
+        curr_pos = connected[0];
+
+        if (connected.size() > 1) { // fork
+            trace_state(connected[1]);
+        }
     }
 }
 
